package org.jeecg.modules.adweb.seo.service.dataforseo;

import io.github.dataforseo.client.ApiClient;
import io.github.dataforseo.client.ApiException;
import io.github.dataforseo.client.api.SerpApi;
import io.github.dataforseo.client.auth.HttpBasicAuth;
import io.github.dataforseo.client.model.*;

import jakarta.annotation.PostConstruct;

import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.adweb.common.util.DateUtil;
import org.jeecg.modules.adweb.common.util.ListUtil;
import org.jeecg.modules.adweb.seo.entity.SeoKeywords;
import org.jeecg.modules.adweb.seo.entity.SeoKeywordsSerp;
import org.jeecg.modules.adweb.seo.mapper.SeoKeywordsMapper;
import org.jeecg.modules.adweb.seo.service.ISeoKeywordsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author wfansh
 */
@Slf4j
@Service
public class DataForSEOService {

    @Value("${dataforseo.username}")
    private String username;

    @Value("${dataforseo.password}")
    private String password;

    @Value("${dataforseo.api-path}")
    private String apiPath;

    @Autowired private SeoKeywordsMapper seoKeywordsMapper;

    @Autowired private ISeoKeywordsService seoKeywordsService;

    private SerpApi serpApi;

    @PostConstruct
    private void init() {
        ApiClient defaultClient = io.github.dataforseo.client.Configuration.getDefaultApiClient();
        defaultClient.setBasePath(apiPath);
        // HTTP超时 - 30秒
        defaultClient.setConnectTimeout(30 * 1000);

        // API认证方式 - basicAuth
        HttpBasicAuth basicAuth = (HttpBasicAuth) defaultClient.getAuthentication("basicAuth");
        basicAuth.setUsername(username);
        basicAuth.setPassword(password);

        this.serpApi = new SerpApi(defaultClient);
    }

    /**
     * 从DataForSEO拉取keywords serp数据，同步到{@link SeoKeywordsSerp}表
     *
     * @param keywordType 1 - 指定词; 2 - 长尾词
     */
    public void syncKeywordsSerp(int keywordType, int limit) throws ApiException {
        Date now = new Date();

        // 1. 查询待更新keywords
        List<SeoKeywords> seoKeywordsList =
                seoKeywordsMapper.getKeywordsToSerp(keywordType).subList(0, limit);
        if (ListUtil.isEmpty(seoKeywordsList)) {
            log.info("没有待Serp查询的关键词");
            return;
        }

        // 2. 发送DataForSEO Serp查询请求
        //        List<SerpGoogleOrganicLiveAdvancedRequestInfo> serpTasks =
        //                seoKeywordsList.stream()
        //                        .map(
        //                                seoKeyword -> {
        //                                    SerpGoogleOrganicLiveAdvancedRequestInfo serpTask =
        //                                            new
        // SerpGoogleOrganicLiveAdvancedRequestInfo();
        //                                    serpTask.setKeyword(seoKeyword.getKeywords());
        //                                    serpTask.setTag(Integer.toString(seoKeyword.getId()));
        //                                    return serpTask;
        //                                })
        //                        .toList();
        //
        //        SerpGoogleOrganicLiveAdvancedResponseInfo serpResults =
        //                serpApi.googleOrganicLiveAdvanced(serpTasks);

        // 3. 更新SeoKeywords表
        seoKeywordsList.forEach(
                seoKeyword -> {
                    // TODO: why?
                    seoKeyword.setTimerLastSearchTime(DateUtil.getTodayZeroTime(now));
                    // On search.
                    seoKeyword.setSearchStatus(1);
                });
        seoKeywordsService.updateBatchById(seoKeywordsList);
        log.info(
                "{}个关键词serp查询任务创建完成 {}",
                seoKeywordsList.size(),
                seoKeywordsList.stream().map(SeoKeywords::getId).toList());
    }
}
